import java.util.List;
/**
 * Hunters have no maximum age and neither feed nor breed. 
 * It goes around randomly in the field and shoots in random direcetions
 * @author Poul Henriksen
 * @version 2005.03.14
 */
public class Hunter implements Actor
{
    private static final int SHOTS_PER_ROUND = 6;
    
    private Location location;
    
    /**
     * Constructor for objects of class Hunter
     */
    public Hunter()
    {
    }

    public Location getLocation() 
    {
        return location;
    }
    
    
    /**
     * Set the hunter's location.
     * @param row The vertical coordinate of the location.
     * @param col The horizontal coordinate of the location.
     */
    public void setLocation(int row, int col)
    {
        this.location = new Location(row, col);
    }

    /**
     * Set the hunter's location.
     * @param location The hunter's location.
     */
    public void setLocation(Location location)
    {
        this.location = location;
    }
    
    /**
     * This is what the hunter does most of the time: it walks around.
     * @param currentField The field currently occupied.
     * @param updatedField The field to transfer to.
     * @param newAnimals A list to add newly born hunters to.
     */
    public void act(Field currentField, Field updatedField, List newAnimals)
    {
            // move randomly
            Location newLocation = updatedField.getRandomLocation();
            
            Actor actorHere = currentField.getObjectAt(newLocation);
            if(actorHere instanceof Hunter) {
                //there is already a hunter here - we stay where we are
                
            }
            else if (actorHere instanceof Animal) {
                //animals we kill
                ((Animal) actorHere).setDead();
                setLocation(newLocation);
                fireAround(currentField);
                updatedField.place(this);  // sets location
            }
    }
    
    private void fireAround(Field field) {        
        for(int i = 0; i < SHOTS_PER_ROUND; i++) {
             Location randomLocation = field.randomAdjacentLocation(location);
             Actor actor = field.getObjectAt(randomLocation);
             if(actor instanceof Animal) {
                 ((Animal) actor).setDead();
             }
        }  
    }    
}
